"""
This code retrieves daily adjusted stock price data for a list of stock symbols
 using the Alpha Vantage API. It saves the data as JSON files, performs data 
 analysis and preprocessing, and generates line plots of cumulative returns 
 over time for each symbol. The plots are displayed using Matplotlib.

The code follows the following steps:
1. Define the stock symbols and API key.
2. Retrieve stock data from the API and save it as JSON files.
3. Load the stock price data from the JSON files.
4. Extract relevant data, convert dates to datetime objects, and calculate 
daily and cumulative returns.
5. Visualize the data by plotting cumulative returns over time for each symbol.
6. Display the plots using Matplotlib's `plt.show()` function.

Note: Before running the code, make sure to replace the API key with your own 
Alpha Vantage API key.

Author: Abraham Reines
Date Created: May 25, 2023
"""
import requests
import json
import matplotlib.pyplot as plt
from matplotlib.dates import DateFormatter
from datetime import datetime

# Step 1: Define stock symbols and API key
symbols = ['AAPL', 'GOOGL', 'MSFT']  # Example list of stock symbols
api_key = '2EGR0ONGHDOTMKYM'  # Replace with your Alpha Vantage API key

# Step 2: Retrieve stock data and save as JSON files
for symbol in symbols:
    url = f'https://www.alphavantage.co/query?function=TIME_SERIES_DAILY_ADJUSTED&symbol={symbol}&outputsize=full&apikey={api_key}'
    response = requests.get(url)
    data = response.json()
    with open(f'{symbol}_data.json', 'w') as file:
        json.dump(data, file)

# Step 3: Data Analysis and Preprocessing
for symbol in symbols:
    # Load the stock price data from JSON
    with open(f'{symbol}_data.json', 'r') as file:
        data = json.load(file)

    # Extract relevant data from the JSON response
    time_series = data['Time Series (Daily)']
    dates = sorted(time_series.keys())

    # Convert dates to datetime objects
    dates = [datetime.strptime(date, '%Y-%m-%d') for date in dates]

    adjusted_close_prices = [float(time_series[date.strftime('%Y-%m-%d')]['5. adjusted close']) for date in dates]

    # Calculate daily returns
    daily_returns = [(adjusted_close_prices[i] / adjusted_close_prices[i-1]) - 1 for i in range(1, len(adjusted_close_prices))]

    # Calculate cumulative returns
    cumulative_returns = [1]
    for i in range(1, len(daily_returns)):
        cumulative_return = (1 + daily_returns[i]) * cumulative_returns[i-1]
        cumulative_returns.append(cumulative_return)

    # Visualize the data
    plt.figure(figsize=(12, 6))

    # Line plot of cumulative returns with all data points
    plt.plot(dates[:len(cumulative_returns)], cumulative_returns, marker='o', markersize=6, label='Cumulative Returns', color='purple', linewidth=2)

    # Set x-axis tick labels and format
    plt.xticks(rotation=45, ha='right')
    date_format = DateFormatter("%Y-%m-%d")
    plt.gca().xaxis.set_major_formatter(date_format)

    # Set y-axis tick labels and format
    plt.gca().yaxis.set_major_formatter('{:.0%}'.format)

    # Set plot title and labels
    plt.title(f'{symbol} Stock Cumulative Returns', fontsize=16, fontweight='bold')
    plt.xlabel('Date', fontsize=12)
    plt.ylabel('Cumulative Returns', fontsize=12)

    # Add grid lines
    plt.grid(True, linestyle='--', linewidth=0.5, alpha=0.7)

    # Add legend
    plt.legend()

    # Adjust plot spacing
    plt.tight_layout()

    plt.show()
